@@ -30,18 +30,22 @@ TASK_LIST = ['MakeBread','EatBread','BuildHouse','ChopTree','ChopRock','GoToHous
 
 
 class CraftingWorldEnv(gym.GoalEnv):
-    """Custom Crafting that follows gym interface"""
+    """Custom Crafting environment that follows the gym interface pattern
+    """
     metadata = {'render.modes': ['human', 'Non']}
 
-    def __init__(self, size=(10, 10), fixed_init_state=None, fixed_goal=None, tasks_to_ignore=None, store_gif=False, max_steps=300, task_list=TASK_LIST):
+    def __init__(self, size=(10, 10), fixed_init_state=None, fixed_goal=None, tasks_to_ignore=None, store_gif=False, render_flipping=False, max_steps=300, task_list=TASK_LIST, pos_rewards = False):
         """
-        initialise the environment, change the following args to create a custom environment
+        change the following parameters to create a custom environment
+
         :param size: size of the grid world
-        :param fixed_init_state: an initial observation to reset to if desired
+        :param fixed_init_state: a fixed initial observation to reset to
         :param fixed_goal: a fixed list of tasks for the agent to achieve
-        :param tasks_to_ignore: a list of tasks that won't affect the reward regardless of completion
+        :param tasks_to_ignore: a list of tasks to ignore when calculating reward
         :param store_gif: whether or not to store every episode as a gif in a /renders/ subdirectory
+        :param render_flipping: set to true if only specific episodes need to be rendered
         :param max_steps: max number of steps the agent can take
+        :param task_list: list of possible tasks
         """
         self.metadata = {'render.modes': ['human', 'Non']}
 
@@ -53,6 +57,7 @@ class CraftingWorldEnv(gym.GoalEnv):
         if tasks_to_ignore:
             for task in tasks_to_ignore:
                 self.task_list.remove(task)
+        self.pos_rewards = pos_rewards
 
         self.observation_space = spaces.Dict({'observation':spaces.Box(low=0, high=1,
                                             shape=(self.num_rows, self.num_cols, len(OBJECTS)+1+len(PICKUPABLE)),
@@ -74,13 +79,12 @@ class CraftingWorldEnv(gym.GoalEnv):
         else:
             self.desired_goal = np.random.randint(2, size=(1, len(self.task_list)))
 
-
         self.achieved_goal = self.observation_space.spaces['achieved_goal'].low
 
         self.fixed_init_state = fixed_init_state
 
-        if self.fixed_init_state:
-            self.obs = self.fixed_init_state
+        if self.fixed_init_state is not None:
+            self.obs = copy.deepcopy(self.fixed_init_state)
             self.agent_pos = coord(int(np.where(np.argmax(self.obs, axis=2) == 8)[0]),
                                    int(np.where(np.argmax(self.obs, axis=2) == 8)[1]), self.num_rows - 1,
                                    self.num_cols - 1)
@@ -95,22 +99,30 @@ class CraftingWorldEnv(gym.GoalEnv):
 
         self.action_space = spaces.Discrete(len(self.ACTIONS))
 
+        self.reward = self.calculate_rewards()
+
         self.store_gif = store_gif
+
+        self.render_flipping = render_flipping
         self.env_id = None
         self.fig, self.ax, self.ims = None, None, None
         self.ep_no = 0
         self.step_num = 0
+        if self.store_gif:
+            self.allow_gif_storage()
 
-    def reset(self):
+    def reset(self, render_next=False):
         """
         reset the environment
         """
-
         # save episode as gif
         if self.store_gif is True and self.step_num != 0:
             # print('debug_final', len(self.ims))
             anim = animation.ArtistAnimation(self.fig, self.ims, interval=100000, blit=False, repeat_delay=1000)
-            anim.save('renders/env{}/episode_{}.gif'.format(self.env_id, self.ep_no), writer=animation.PillowWriter(), dpi=100)
+            anim.save('renders/env{}/episode_{}_({}).gif'.format(self.env_id, self.ep_no, self.step_num), writer=animation.PillowWriter(), dpi=100)
+
+        if self.render_flipping is True:
+            self.store_gif = render_next
 
         if self.fixed_goal:
             self.desired_goal = np.zeros(shape=(1, len(self.task_list)), dtype=int)
@@ -121,8 +133,8 @@ class CraftingWorldEnv(gym.GoalEnv):
 
         self.achieved_goal = self.observation_space.spaces['achieved_goal'].low
 
-        if self.fixed_init_state:
-            self.obs = self.fixed_init_state
+        if self.fixed_init_state is not None:
+            self.obs = copy.deepcopy(self.fixed_init_state)
             self.agent_pos = coord(int(np.where(np.argmax(self.obs, axis=2) == 8)[0]),
                                    int(np.where(np.argmax(self.obs, axis=2) == 8)[1]), self.num_rows - 1,
                                    self.num_cols - 1)
@@ -134,23 +146,28 @@ class CraftingWorldEnv(gym.GoalEnv):
 
         self.init_observation = copy.deepcopy(self.observation)
 
+        self.reward = self.calculate_rewards()
+
         if self.step_num != 0:  # don't increment episode number if resetting after init
             self.ep_no += 1
 
         self.step_num = 0
 
         # reset gif
+        plt.close('all')
         if self.store_gif:
-            if self.fig is None:
-                self.fig, self.ax = plt.subplots(1)
-            else:
-                plt.clf()
+            # if self.fig is None:
+            #     self.fig, self.ax = plt.subplots(1)
+            # else:
+            #     plt.clf()
+            self.fig, self.ax = plt.subplots(1)
             self.ims = []
             self.__render_gif()
 
     def step(self, action):
         """
         take a step within the environment
+
         :param action: integer value within the action_space range
         :return: observations, reward, done, debugging info
         """
@@ -189,6 +206,16 @@ class CraftingWorldEnv(gym.GoalEnv):
         else:
             self.__move_agent(action_value)
 
+        task_success = self.eval_tasks()
+        self.achieved_goal = self.task_one_hot(task_success)
+        self.observation = {'observation': self.obs, 'desired_goal': self.desired_goal,
+                            'achieved_goal': self.achieved_goal}
+        observation = self.observation
+        self.reward = self.calculate_rewards()
+        reward = self.reward
+        reward_lim = 0 if self.pos_rewards is False else np.sum(self.desired_goal)
+        done = False if self.step_num < self.max_steps or reward == reward_lim else True
+
         # render if required
         if self.store_gif is True:
             if type(action_value) == coord:
@@ -196,12 +223,6 @@ class CraftingWorldEnv(gym.GoalEnv):
             else:
                 self.__render_gif(action_value)
 
-        task_success = self.eval_tasks()
-        self.achieved_goal = self.task_one_hot(task_success)
-        observation = self.observation
-        reward = self.calculate_rewards()
-        done = False if self.step_num < self.max_steps or reward == 0 else True
-
         return observation, reward, done, {"task_success":  task_success, "desired_goal": self.desired_goal}
 
     def __move_agent(self, action):
@@ -217,7 +238,6 @@ class CraftingWorldEnv(gym.GoalEnv):
 
         :param action: one of the movement actions, stored as a coordinate object. coordinate class makes it easier to ensure agent doesn't move outside the grid
         """
-
         new_pos = self.agent_pos + action
 
         if new_pos == self.agent_pos:  # agent is at an edge coordinate, so can't move in that direction
@@ -326,7 +346,7 @@ class CraftingWorldEnv(gym.GoalEnv):
 Episode {}: step {} - action choice: {}
 Desired Goals: {}""".format(self.ep_no,self.step_num,action_label,desired_goals)
 
-        bottom_text = "Achieved Goals: {}".format(achieved_goals)
+        bottom_text = "Achieved Goals: {}\nd_g: {}\na_g: {},   r: {}".format(achieved_goals,self.desired_goal,self.achieved_goal, self.reward)
         ttl = plt.text(0.00, 1.01, title_str, horizontalalignment='left',
                        verticalalignment='bottom', transform=self.ax.transAxes)
         txt = plt.text(0.00, -0.02, bottom_text, horizontalalignment='left',
@@ -349,7 +369,6 @@ Desired Goals: {}""".format(self.ep_no,self.step_num,action_label,desired_goals)
         :return obs: a sample observation
         :return agent_position: position of the agent within the observation
         """
-
         objects = [_ for _ in range(1,10)]
         objects = [self.one_hot(i - 1) for i in objects]
         grid = objects + [[0 for _ in range(self.observation_space.spaces['observation'].shape[2])]
@@ -365,6 +384,7 @@ Desired Goals: {}""".format(self.ep_no,self.step_num,action_label,desired_goals)
         return state, agent_position
 
     def eval_tasks(self):
+        # TODO: eval_tasks is not efficient (dictcomps are slow) - try to speed this up
         task_success = {}
         init_objects = {obj: self.get_objects(code, self.init_observation['observation']) for code, obj in enumerate(OBJECTS)}
         final_objects = {obj: self.get_objects(code, self.obs) for code, obj in enumerate(OBJECTS)}
@@ -375,7 +395,7 @@ Desired Goals: {}""".format(self.ep_no,self.step_num,action_label,desired_goals)
         task_success['BuildHouse'] = len(final_objects['house']) > len(init_objects['house'])
         task_success['ChopTree'] = len(final_objects['tree']) < len(init_objects['tree'])
         task_success['ChopRock'] = len(final_objects['rock']) < len(init_objects['rock'])
-        task_success['GoToHouse'] = (self.agent_pos.row, self.agent_pos.col) in final_objects['house']
+        task_success['GoToHouse'] = list((self.agent_pos.row, self.agent_pos.col)) in final_objects['house']
         task_success['MoveAxe'] = final_objects['axe'] != init_objects['axe']
         task_success['MoveHammer'] = final_objects['hammer'] != init_objects['hammer']
         task_success['MoveSticks'] = False in [stick in init_objects['sticks'] for stick in final_objects['sticks']]
@@ -392,6 +412,8 @@ Desired Goals: {}""".format(self.ep_no,self.step_num,action_label,desired_goals)
 
     def calculate_rewards(self):
         error = np.sum(np.square(self.achieved_goal - self.desired_goal))
+        if self.pos_rewards is True:
+            return np.sum(self.desired_goal)-error
         return -error
 
     def get_objects(self, code, state):
